/****************************************************************************
** Meta object code from reading C++ file 'rpibootthread.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../rpi-imager/src/rpibootthread.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rpibootthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13RpibootThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto RpibootThread::qt_create_metaobjectdata<qt_meta_tag_ZN13RpibootThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RpibootThread",
        "success",
        "",
        "error",
        "msg",
        "preparationStatusUpdate",
        "progressChanged",
        "current",
        "total",
        "fastbootDeviceReady",
        "fastbootId",
        "eventFirmwareSetup",
        "durationMs",
        "metadata",
        "eventRpibootProtocol",
        "eventFastbootWait"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'success'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QString)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'preparationStatusUpdate'
        QtMocHelpers::SignalData<void(QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'progressChanged'
        QtMocHelpers::SignalData<void(quint64, quint64)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 7 }, { QMetaType::ULongLong, 8 },
        }}),
        // Signal 'fastbootDeviceReady'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'eventFirmwareSetup'
        QtMocHelpers::SignalData<void(quint32, bool, QString)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 }, { QMetaType::Bool, 1 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'eventRpibootProtocol'
        QtMocHelpers::SignalData<void(quint32, bool, QString)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 }, { QMetaType::Bool, 1 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'eventFastbootWait'
        QtMocHelpers::SignalData<void(quint32, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 }, { QMetaType::Bool, 1 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RpibootThread, qt_meta_tag_ZN13RpibootThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RpibootThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13RpibootThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13RpibootThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13RpibootThreadE_t>.metaTypes,
    nullptr
} };

void RpibootThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RpibootThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->success(); break;
        case 1: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->preparationStatusUpdate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->progressChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 4: _t->fastbootDeviceReady((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->eventFirmwareSetup((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->eventRpibootProtocol((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 7: _t->eventFastbootWait((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)()>(_a, &RpibootThread::success, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(QString )>(_a, &RpibootThread::error, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(QString )>(_a, &RpibootThread::preparationStatusUpdate, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(quint64 , quint64 )>(_a, &RpibootThread::progressChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(const QString & )>(_a, &RpibootThread::fastbootDeviceReady, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(quint32 , bool , QString )>(_a, &RpibootThread::eventFirmwareSetup, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(quint32 , bool , QString )>(_a, &RpibootThread::eventRpibootProtocol, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (RpibootThread::*)(quint32 , bool )>(_a, &RpibootThread::eventFastbootWait, 7))
            return;
    }
}

const QMetaObject *RpibootThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RpibootThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13RpibootThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int RpibootThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void RpibootThread::success()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void RpibootThread::error(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RpibootThread::preparationStatusUpdate(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void RpibootThread::progressChanged(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void RpibootThread::fastbootDeviceReady(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void RpibootThread::eventFirmwareSetup(quint32 _t1, bool _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void RpibootThread::eventRpibootProtocol(quint32 _t1, bool _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void RpibootThread::eventFastbootWait(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
