/*
 * SPDX-License-Identifier: Apache-2.0
 * Copyright (C) 2025 Raspberry Pi Ltd
 */

#include "imageadvancedoptions.h"

// This file is required for Q_NAMESPACE to work properly
// The moc system needs a .cpp file to generate the staticMetaObject

