/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x2,0xaa,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xed,0x9,0x5,0x15,0x0,0x56,0x5c,0x50,0x1e,0x30,0x73,0x73,
  0x1d,0xc3,0x8a,0x5b,0x57,0xf6,0x7e,0xc9,0x6a,0x4a,0xf8,0xa0,0xbf,0xdb,0xf7,0xeb,
  0xdb,0x3d,0x90,0x58,0x73,0xa8,0x9,0x3,0x43,0x8a,0xa,0x4a,0x0,0x48,0x0,0x48,
  0x0,0xdb,0x52,0x11,0xf1,0xdd,0x28,0x2e,0x2a,0xe6,0xad,0xf4,0x76,0xf5,0x34,0x10,
  0x10,0x20,0x8e,0x3d,0xbd,0x75,0x6e,0x25,0xe8,0x6d,0xbf,0x55,0x4f,0x19,0x10,0x45,
  0x51,0x6f,0xa2,0x2,0x14,0x8f,0x79,0x84,0x0,0xd,0x53,0x2b,0xd2,0x11,0x88,0xa9,
  0x77,0x78,0xa,0xc0,0xcd,0x6d,0x4,0x1a,0x50,0x18,0xa1,0xc5,0x3c,0x3d,0xbc,0x1a,
  0x10,0x5f,0xa5,0x5b,0x28,0x94,0x40,0x6f,0x16,0xb5,0x2e,0xd,0x97,0xb1,0xda,0x8,
  0x81,0xfd,0xa8,0xac,0xcb,0x11,0xfc,0x56,0x1b,0x3e,0x20,0x34,0x5c,0x71,0x5b,0x79,
  0xe1,0x25,0xd5,0xd3,0x17,0xd3,0xb2,0x94,0x35,0x31,0xbf,0x51,0x71,0x3b,0xcc,0x9,
  0x79,0xbb,0x64,0xd3,0xe4,0x18,0xde,0x49,0x41,0xca,0x5b,0x65,0x56,0x17,0xf,0x79,
  0x9a,0x19,0x76,0x9d,0x71,0x8e,0x46,0x84,0x7b,0xf3,0x35,0x45,0x4c,0x45,0x54,0xcc,
  0x8f,0x76,0xaa,0x92,0x65,0x7a,0x74,0xae,0x95,0x66,0x6d,0xfc,0xd6,0x26,0xa6,0xe3,
  0x92,0x93,0xb7,0xaa,0xda,0x76,0x4d,0xb4,0xe7,0x93,0xd6,0x14,0xb2,0x4a,0xa5,0x4,
  0x5a,0x70,0x21,0xe7,0xe4,0xd4,0x34,0xb1,0x17,0xb7,0xe7,0x6d,0x9e,0x3d,0x68,0xd8,
  0x33,0x12,0x75,0xc9,0xdb,0x6e,0xeb,0xe0,0x16,0x11,0xf7,0xfb,0x5b,0xe8,0x96,0x9d,
  0xf3,0x0,0xaf,0xcd,0xde,0x3e,0xbd,0x4a,0xab,0x7a,0x19,0x80,0x2a,0x5f,0x3a,0xae,
  0x33,0x32,0xc,0x35,0x32,0x3e,0xa2,0xfd,0x92,0xd,0x17,0x37,0xf1,0xb7,0xd5,0xd3,
  0xd6,0xc8,0xfe,0xde,0x4a,0xda,0xbb,0x87,0x47,0x8b,0x31,0x9f,0x31,0xc3,0x7b,0x9d,
  0x62,0xe2,0xf9,0x2d,0xd9,0xf2,0xa3,0xd5,0xf9,0x4,0xf5,0xc,0x18,0xb7,0xcd,0x96,
  0x74,0x97,0xaf,0x79,0x15,0x71,0x9b,0x29,0x27,0xfe,0x69,0xf6,0xdb,0x80,0xa8,0xa8,
  0x81,0xca,0xcc,0x8c,0x8,0x69,0x52,0x90,0x42,0x1a,0x3,0x40,0x44,0x20,0x48,0x33,
  0xa9,0x3,0x52,0x38,0x40,0x10,0xe5,0x18,0x8,0x5,0x29,0x86,0x40,0x22,0x50,0x84,
  0x13,0x19,0x59,0xa2,0x5b,0x19,0x3,0xb9,0x7d,0x16,0x5d,0x8,0xff,0xfe,0x94,0x9e,
  0x1e,0x94,0xea,0x21,0xb7,0x71,0x7e,0x47,0xa6,0x85,0x10,0xea,0xb,0x2e,0xec,0xd7,
  0x98,0x28,0xb2,0x34,0x66,0x5d,0xa4,0x1c,0x67,0xcd,0x49,0x68,0x19,0xab,0xb6,0x98,
  0x73,0xed,0xb7,0xb4,0x77,0x4d,0xe6,0x9e,0x9e,0x52,0x81,0xb1,0xa6,0x60,0xb7,0xf3,
  0xc3,0xe6,0xbf,0xdc,0x26,0x20,0x26,0x4f,0xf7,0xb6,0x24,0x52,0xa,0xa8,0xe7,0x57,
  0xa,0xa3,0xa0,0x24,0x64,0x4c,0x9e,0x4d,0xaa,0x97,0xc0,0xf,0x65,0x88,0x65,0x81,
  0x8a,0x41,0xc,0x9e,0x8f,0xd2,0xa4,0x33,0xe4,0xd3,0x63,0xd,0xef,0xec,0x4b,0xea,
  0x82,0xd8,0x73,0x0,0xd2,0x19,0x12,0xf1,0x5d,0x8c,0x20,0x4,0xae,0x83,0x3e,0xb7,
  0xc5,0xba,0xaf,0x97,0x8c,0x19,0xd7,0x6,0x35,0x51,0xef,0xbf,0xee,0xdc,0xe,0x4b,
  0x2f,0x3c,0xb,0x12,0x64,0x30,0x8c,0x63,0x82,0x9f,0x34,0xc,0xe,0xc4,0x9d,0xa9,
  0xe2,0x55,0x88,0xb6,0x19,0xc0,0xc1,0x47,0xbf,0x3d,0x57,0x41,0x4a,0x93,0x3f,0x91,
  0xc5,0x31,0xb4,0x70,0x19,0x92,0x34,0x3c,0xab,0xf7,0x9c,0x86,0xfd,0xaa,0x4,0x36,
  0xda,0x18,0x74,0xc1,0x53,0xa,0xd8,0x22,0xc2,0x92,0x2,0x3c,0x42,0x61,0x5c,0x89,
  0x43,0x4c,0x28,0xb6,0x5e,0x7,0xad,0xc2,0x3a,0x6,0x82,0x2f,0x57,0xbf,0xfc,0xb9,
  0x8b,0xad,0x7b,0x23,0x3,0xca,0xb2,0xcc,0xe2,0x31,0x5,0xe1,0xa,0x4,0xd,0xff,
  0xb4,0xaf,0x30,0x57,0x20,0x5,0x23,0xda,0x92,0xc9,0x8a,0xf9,0x12,0x90,0x2e,0xba,
  0x71,0x73,0x4a,0x2c,0x1,0x5b,0x15,0xc5,0x26,0x1d,0x8c,0x11,0x19,0x5c,0x62,0xa3,
  0xc3,0x85,0x40,0x63,0xda,0x13,0xda,0x24,0x81,0x61,0x96,0xa1,0x47,0x7d,0x3c,0xcf,
  0x4b,0xb0,0xb0,0xa0,0x38,0xa5,0x38,0xe6,0x43,0xae,0xf5,0xb0,0xbe,0x9a,0x7e,0xa5,
  0xde,0x88,0x64,0x55,0xb5,0xe7,0xe3,0xcf,0x2a,
  
};

static const unsigned char qt_resource_name[] = {
  // qt
  0x0,0x2,
  0x0,0x0,0x7,0x84,
  0x0,0x71,
  0x0,0x74,
    // qml
  0x0,0x3,
  0x0,0x0,0x78,0x3c,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,
    // RpiImager
  0x0,0x9,
  0xe,0x3,0xcf,0x2,
  0x0,0x52,
  0x0,0x70,0x0,0x69,0x0,0x49,0x0,0x6d,0x0,0x61,0x0,0x67,0x0,0x65,0x0,0x72,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,
  // :/qt
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,
  // :/qt/qml
  0x0,0x0,0x0,0xa,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,
  // :/qt/qml/RpiImager
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,
  // :/qt/qml/RpiImager/qmldir
  0x0,0x0,0x0,0x2e,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_RpiImager)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_RpiImager)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_RpiImager)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_RpiImager)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_RpiImager)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_RpiImager)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
