/****************************************************************************
** Meta object code from reading C++ file 'cachemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../rpi-imager/src/cachemanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cachemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12CacheManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto CacheManager::qt_create_metaobjectdata<qt_meta_tag_ZN12CacheManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CacheManager",
        "cacheVerificationComplete",
        "",
        "isValid",
        "diskSpaceCheckComplete",
        "availableBytes",
        "cacheOperationsReady",
        "cacheVerificationProgress",
        "bytesProcessed",
        "totalBytes",
        "cacheInvalidated",
        "cacheFileUpdated",
        "uncompressedHash",
        "onVerificationComplete",
        "fileName",
        "expectedHash",
        "computedHash",
        "onDiskSpaceCheckComplete",
        "directory",
        "isCached",
        "hasPotentialCache",
        "getCacheFilePath"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cacheVerificationComplete'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'diskSpaceCheckComplete'
        QtMocHelpers::SignalData<void(qint64)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 5 },
        }}),
        // Signal 'cacheOperationsReady'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cacheVerificationProgress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 }, { QMetaType::LongLong, 9 },
        }}),
        // Signal 'cacheInvalidated'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cacheFileUpdated'
        QtMocHelpers::SignalData<void(const QByteArray &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 12 },
        }}),
        // Slot 'onVerificationComplete'
        QtMocHelpers::SlotData<void(bool, const QString &, const QByteArray &, const QByteArray &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::QString, 14 }, { QMetaType::QByteArray, 15 }, { QMetaType::QByteArray, 16 },
        }}),
        // Slot 'onDiskSpaceCheckComplete'
        QtMocHelpers::SlotData<void(qint64, const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 5 }, { QMetaType::QString, 18 },
        }}),
        // Method 'isCached'
        QtMocHelpers::MethodData<bool(const QByteArray &) const>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Method 'hasPotentialCache'
        QtMocHelpers::MethodData<bool(const QByteArray &) const>(20, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Method 'getCacheFilePath'
        QtMocHelpers::MethodData<QString(const QByteArray &) const>(21, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CacheManager, qt_meta_tag_ZN12CacheManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CacheManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CacheManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CacheManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12CacheManagerE_t>.metaTypes,
    nullptr
} };

void CacheManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CacheManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cacheVerificationComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->diskSpaceCheckComplete((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 2: _t->cacheOperationsReady(); break;
        case 3: _t->cacheVerificationProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 4: _t->cacheInvalidated(); break;
        case 5: _t->cacheFileUpdated((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 6: _t->onVerificationComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4]))); break;
        case 7: _t->onDiskSpaceCheckComplete((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: { bool _r = _t->isCached((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->hasPotentialCache((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->getCacheFilePath((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CacheManager::*)(bool )>(_a, &CacheManager::cacheVerificationComplete, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheManager::*)(qint64 )>(_a, &CacheManager::diskSpaceCheckComplete, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheManager::*)()>(_a, &CacheManager::cacheOperationsReady, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheManager::*)(qint64 , qint64 )>(_a, &CacheManager::cacheVerificationProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheManager::*)()>(_a, &CacheManager::cacheInvalidated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheManager::*)(const QByteArray & )>(_a, &CacheManager::cacheFileUpdated, 5))
            return;
    }
}

const QMetaObject *CacheManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CacheManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CacheManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CacheManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void CacheManager::cacheVerificationComplete(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CacheManager::diskSpaceCheckComplete(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CacheManager::cacheOperationsReady()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CacheManager::cacheVerificationProgress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void CacheManager::cacheInvalidated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void CacheManager::cacheFileUpdated(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN23CacheVerificationWorkerE_t {};
} // unnamed namespace

template <> constexpr inline auto CacheVerificationWorker::qt_create_metaobjectdata<qt_meta_tag_ZN23CacheVerificationWorkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CacheVerificationWorker",
        "verificationComplete",
        "",
        "isValid",
        "fileName",
        "expectedHash",
        "computedHash",
        "diskSpaceCheckComplete",
        "availableBytes",
        "directory",
        "verificationProgress",
        "bytesProcessed",
        "totalBytes",
        "verifyCacheFile",
        "checkDiskSpace"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'verificationComplete'
        QtMocHelpers::SignalData<void(bool, const QString &, const QByteArray &, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::QString, 4 }, { QMetaType::QByteArray, 5 }, { QMetaType::QByteArray, 6 },
        }}),
        // Signal 'diskSpaceCheckComplete'
        QtMocHelpers::SignalData<void(qint64, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'verificationProgress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 11 }, { QMetaType::LongLong, 12 },
        }}),
        // Slot 'verifyCacheFile'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Slot 'checkDiskSpace'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CacheVerificationWorker, qt_meta_tag_ZN23CacheVerificationWorkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CacheVerificationWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23CacheVerificationWorkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23CacheVerificationWorkerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23CacheVerificationWorkerE_t>.metaTypes,
    nullptr
} };

void CacheVerificationWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CacheVerificationWorker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->verificationComplete((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4]))); break;
        case 1: _t->diskSpaceCheckComplete((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->verificationProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 3: _t->verifyCacheFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 4: _t->checkDiskSpace(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CacheVerificationWorker::*)(bool , const QString & , const QByteArray & , const QByteArray & )>(_a, &CacheVerificationWorker::verificationComplete, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheVerificationWorker::*)(qint64 , const QString & )>(_a, &CacheVerificationWorker::diskSpaceCheckComplete, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CacheVerificationWorker::*)(qint64 , qint64 )>(_a, &CacheVerificationWorker::verificationProgress, 2))
            return;
    }
}

const QMetaObject *CacheVerificationWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CacheVerificationWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23CacheVerificationWorkerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CacheVerificationWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void CacheVerificationWorker::verificationComplete(bool _t1, const QString & _t2, const QByteArray & _t3, const QByteArray & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void CacheVerificationWorker::diskSpaceCheckComplete(qint64 _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void CacheVerificationWorker::verificationProgress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
