/****************************************************************************
** Meta object code from reading C++ file 'downloadextractthread.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../rpi-imager/src/downloadextractthread.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'downloadextractthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21DownloadExtractThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto DownloadExtractThread::qt_create_metaobjectdata<qt_meta_tag_ZN21DownloadExtractThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DownloadExtractThread",
        "downloadProgressChanged",
        "",
        "now",
        "total",
        "decompressProgressChanged",
        "writeProgressChanged",
        "verifyProgressChanged",
        "eventRingBufferStats",
        "timestampMs",
        "durationMs",
        "metadata",
        "eventPipelineDecompressionTime",
        "totalMs",
        "bytesDecompressed",
        "eventPipelineRingBufferWaitTime",
        "bytesRead",
        "eventWriteRingBufferStats",
        "producerStalls",
        "consumerStalls",
        "producerWaitMs",
        "consumerWaitMs"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'downloadProgressChanged'
        QtMocHelpers::SignalData<void(quint64, quint64)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 3 }, { QMetaType::ULongLong, 4 },
        }}),
        // Signal 'decompressProgressChanged'
        QtMocHelpers::SignalData<void(quint64, quint64)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 3 }, { QMetaType::ULongLong, 4 },
        }}),
        // Signal 'writeProgressChanged'
        QtMocHelpers::SignalData<void(quint64, quint64)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 3 }, { QMetaType::ULongLong, 4 },
        }}),
        // Signal 'verifyProgressChanged'
        QtMocHelpers::SignalData<void(quint64, quint64)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 3 }, { QMetaType::ULongLong, 4 },
        }}),
        // Signal 'eventRingBufferStats'
        QtMocHelpers::SignalData<void(qint64, quint32, QString)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 9 }, { QMetaType::UInt, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'eventPipelineDecompressionTime'
        QtMocHelpers::SignalData<void(quint32, quint64)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::ULongLong, 14 },
        }}),
        // Signal 'eventPipelineRingBufferWaitTime'
        QtMocHelpers::SignalData<void(quint32, quint64)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::ULongLong, 16 },
        }}),
        // Signal 'eventWriteRingBufferStats'
        QtMocHelpers::SignalData<void(quint64, quint64, quint64, quint64)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 18 }, { QMetaType::ULongLong, 19 }, { QMetaType::ULongLong, 20 }, { QMetaType::ULongLong, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DownloadExtractThread, qt_meta_tag_ZN21DownloadExtractThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DownloadExtractThread::staticMetaObject = { {
    QMetaObject::SuperData::link<DownloadThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DownloadExtractThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DownloadExtractThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21DownloadExtractThreadE_t>.metaTypes,
    nullptr
} };

void DownloadExtractThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DownloadExtractThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->downloadProgressChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 1: _t->decompressProgressChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 2: _t->writeProgressChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 3: _t->verifyProgressChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 4: _t->eventRingBufferStats((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 5: _t->eventPipelineDecompressionTime((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 6: _t->eventPipelineRingBufferWaitTime((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 7: _t->eventWriteRingBufferStats((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint64 , quint64 )>(_a, &DownloadExtractThread::downloadProgressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint64 , quint64 )>(_a, &DownloadExtractThread::decompressProgressChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint64 , quint64 )>(_a, &DownloadExtractThread::writeProgressChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint64 , quint64 )>(_a, &DownloadExtractThread::verifyProgressChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(qint64 , quint32 , QString )>(_a, &DownloadExtractThread::eventRingBufferStats, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint32 , quint64 )>(_a, &DownloadExtractThread::eventPipelineDecompressionTime, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint32 , quint64 )>(_a, &DownloadExtractThread::eventPipelineRingBufferWaitTime, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadExtractThread::*)(quint64 , quint64 , quint64 , quint64 )>(_a, &DownloadExtractThread::eventWriteRingBufferStats, 7))
            return;
    }
}

const QMetaObject *DownloadExtractThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DownloadExtractThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DownloadExtractThreadE_t>.strings))
        return static_cast<void*>(this);
    return DownloadThread::qt_metacast(_clname);
}

int DownloadExtractThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DownloadThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void DownloadExtractThread::downloadProgressChanged(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DownloadExtractThread::decompressProgressChanged(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DownloadExtractThread::writeProgressChanged(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void DownloadExtractThread::verifyProgressChanged(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void DownloadExtractThread::eventRingBufferStats(qint64 _t1, quint32 _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void DownloadExtractThread::eventPipelineDecompressionTime(quint32 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void DownloadExtractThread::eventPipelineRingBufferWaitTime(quint32 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void DownloadExtractThread::eventWriteRingBufferStats(quint64 _t1, quint64 _t2, quint64 _t3, quint64 _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
