/****************************************************************************
** Meta object code from reading C++ file 'imageadvancedoptions.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../rpi-imager/src/imageadvancedoptions.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'imageadvancedoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ImageOptionsE_t {};
} // unnamed namespace

template <> constexpr inline auto ImageOptions::qt_create_metaobjectdata<qt_meta_tag_ZN12ImageOptionsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ImageOptions",
        "AdvancedOption",
        "NoAdvancedOptions",
        "IsRpiosCloudInit",
        "EnableSsh",
        "UserDefinedFirstRun",
        "EnableSecureBoot",
        "AdvancedOptions"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AdvancedOption'
        QtMocHelpers::EnumData<AdvancedOption>(1, 1, QMC::EnumFlags{}).add({
            {    2, AdvancedOption::NoAdvancedOptions },
            {    3, AdvancedOption::IsRpiosCloudInit },
            {    4, AdvancedOption::EnableSsh },
            {    5, AdvancedOption::UserDefinedFirstRun },
            {    6, AdvancedOption::EnableSecureBoot },
        }),
        // flag 'AdvancedOptions'
        QtMocHelpers::EnumData<AdvancedOptions>(7, 1, QMC::EnumIsFlag).add({
            {    2, AdvancedOption::NoAdvancedOptions },
            {    3, AdvancedOption::IsRpiosCloudInit },
            {    4, AdvancedOption::EnableSsh },
            {    5, AdvancedOption::UserDefinedFirstRun },
            {    6, AdvancedOption::EnableSecureBoot },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN12ImageOptionsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN12ImageOptionsE =
    ImageOptions::qt_create_metaobjectdata<qt_meta_tag_ZN12ImageOptionsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN12ImageOptionsE =
    qt_staticMetaObjectContent_ZN12ImageOptionsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN12ImageOptionsE =
    qt_staticMetaObjectContent_ZN12ImageOptionsE.relocatingData;

Q_CONSTINIT const QMetaObject ImageOptions::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN12ImageOptionsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN12ImageOptionsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN12ImageOptionsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
