/****************************************************************************
** Meta object code from reading C++ file 'drivelistmodelpollthread.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../rpi-imager/src/drivelistmodelpollthread.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'drivelistmodelpollthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24DriveListModelPollThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto DriveListModelPollThread::qt_create_metaobjectdata<qt_meta_tag_ZN24DriveListModelPollThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DriveListModelPollThread",
        "newDriveList",
        "",
        "std::vector<Drivelist::DeviceDescriptor>",
        "list",
        "scanModeChanged",
        "ScanMode",
        "mode",
        "eventDriveListPoll",
        "durationMs"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newDriveList'
        QtMocHelpers::SignalData<void(std::vector<Drivelist::DeviceDescriptor>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'scanModeChanged'
        QtMocHelpers::SignalData<void(enum ScanMode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'eventDriveListPoll'
        QtMocHelpers::SignalData<void(quint32)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DriveListModelPollThread, qt_meta_tag_ZN24DriveListModelPollThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DriveListModelPollThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DriveListModelPollThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DriveListModelPollThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24DriveListModelPollThreadE_t>.metaTypes,
    nullptr
} };

void DriveListModelPollThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DriveListModelPollThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newDriveList((*reinterpret_cast<std::add_pointer_t<std::vector<Drivelist::DeviceDescriptor>>>(_a[1]))); break;
        case 1: _t->scanModeChanged((*reinterpret_cast<std::add_pointer_t<enum ScanMode>>(_a[1]))); break;
        case 2: _t->eventDriveListPoll((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DriveListModelPollThread::*)(std::vector<Drivelist::DeviceDescriptor> )>(_a, &DriveListModelPollThread::newDriveList, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DriveListModelPollThread::*)(ScanMode )>(_a, &DriveListModelPollThread::scanModeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DriveListModelPollThread::*)(quint32 )>(_a, &DriveListModelPollThread::eventDriveListPoll, 2))
            return;
    }
}

const QMetaObject *DriveListModelPollThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DriveListModelPollThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DriveListModelPollThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int DriveListModelPollThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void DriveListModelPollThread::newDriveList(std::vector<Drivelist::DeviceDescriptor> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DriveListModelPollThread::scanModeChanged(ScanMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DriveListModelPollThread::eventDriveListPoll(quint32 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
