/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<clipboardhelper.h>)
#  include <clipboardhelper.h>
#endif
#if __has_include(<drivelistmodel.h>)
#  include <drivelistmodel.h>
#endif
#if __has_include(<hwlistmodel.h>)
#  include <hwlistmodel.h>
#endif
#if __has_include(<imagewriter.h>)
#  include <imagewriter.h>
#endif
#if __has_include(<oslistmodel.h>)
#  include <oslistmodel.h>
#endif
#if __has_include(<platformhelper.h>)
#  include <platformhelper.h>
#endif
#if __has_include(<urlfmt.h>)
#  include <urlfmt.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_RpiImager()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<ClipboardHelper>("RpiImager", 1);
    qmlRegisterTypesAndRevisions<DriveListModel>("RpiImager", 1);
    qmlRegisterTypesAndRevisions<HWListModel>("RpiImager", 1);
    qmlRegisterTypesAndRevisions<ImageWriter>("RpiImager", 1);
    qmlRegisterEnum<ImageWriter::WriteState>("ImageWriter::WriteState");
    qmlRegisterTypesAndRevisions<OSListModel>("RpiImager", 1);
    qmlRegisterTypesAndRevisions<PlatformHelper>("RpiImager", 1);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    qmlRegisterTypesAndRevisions<UrlFmt>("RpiImager", 1);
    QT_WARNING_POP
    qmlRegisterModule("RpiImager", 1, 0);
}

static const QQmlModuleRegistration rpiImagerRegistration("RpiImager", qml_register_types_RpiImager);
