/****************************************************************************
** Meta object code from reading C++ file 'downloadthread.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../rpi-imager/src/downloadthread.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'downloadthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DownloadThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto DownloadThread::qt_create_metaobjectdata<qt_meta_tag_ZN14DownloadThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DownloadThread",
        "success",
        "",
        "error",
        "msg",
        "cacheFileUpdated",
        "sha256",
        "cacheFileHashUpdated",
        "cacheFileHash",
        "imageHash",
        "finalizing",
        "preparationStatusUpdate",
        "eventDriveUnmount",
        "durationMs",
        "eventDriveUnmountVolumes",
        "eventDriveDiskClean",
        "eventDriveRescan",
        "eventDriveOpen",
        "metadata",
        "eventDriveAuthorization",
        "eventDriveMbrZeroing",
        "eventDirectIOAttempt",
        "attempted",
        "succeeded",
        "currentlyEnabled",
        "errorCode",
        "errorMessage",
        "eventCustomisation",
        "finalSyncStarting",
        "eventFinalSync",
        "eventVerify",
        "writeHash",
        "verifyHash",
        "eventDecompressInit",
        "eventPeriodicSync",
        "bytesWritten",
        "eventImageExtraction",
        "eventPartitionTableWrite",
        "eventFatPartitionSetup",
        "eventDeviceClose",
        "eventNetworkRetry",
        "sleepMs",
        "eventNetworkConnectionStats",
        "eventDeviceIOTimeout",
        "pendingWrites",
        "eventQueueDepthReduction",
        "oldDepth",
        "newDepth",
        "eventDrainAndHotSwap",
        "pendingBefore",
        "syncFallbackActivated",
        "reason",
        "requestWriteRestart",
        "eventWriteTimingBreakdown",
        "totalWriteOps",
        "totalSyscallMs",
        "totalPreHashWaitMs",
        "totalPostHashWaitMs",
        "totalSyncMs",
        "syncCount",
        "eventWriteSizeDistribution",
        "minSizeKB",
        "maxSizeKB",
        "avgSizeKB",
        "totalBytes",
        "writeCount",
        "eventWriteAfterSyncImpact",
        "avgThroughputBeforeSyncKBps",
        "avgThroughputAfterSyncKBps",
        "sampleCount",
        "eventAsyncIOConfig",
        "enabled",
        "supported",
        "queueDepth",
        "pendingAtEnd",
        "eventAsyncIOTiming",
        "totalMs",
        "bottleneckStateChanged",
        "DownloadThread::BottleneckState",
        "state",
        "throughputKBps",
        "asyncWriteProgress",
        "BottleneckState",
        "None",
        "Network",
        "Decompression",
        "Storage",
        "Verifying"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'success'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QString)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'cacheFileUpdated'
        QtMocHelpers::SignalData<void(QByteArray)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 6 },
        }}),
        // Signal 'cacheFileHashUpdated'
        QtMocHelpers::SignalData<void(QByteArray, QByteArray)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 8 }, { QMetaType::QByteArray, 9 },
        }}),
        // Signal 'finalizing'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preparationStatusUpdate'
        QtMocHelpers::SignalData<void(QString)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'eventDriveUnmount'
        QtMocHelpers::SignalData<void(quint32, bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventDriveUnmountVolumes'
        QtMocHelpers::SignalData<void(quint32, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventDriveDiskClean'
        QtMocHelpers::SignalData<void(quint32, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventDriveRescan'
        QtMocHelpers::SignalData<void(quint32, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventDriveOpen'
        QtMocHelpers::SignalData<void(quint32, bool, QString)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'eventDriveAuthorization'
        QtMocHelpers::SignalData<void(quint32, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventDriveMbrZeroing'
        QtMocHelpers::SignalData<void(quint32, bool, QString)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'eventDirectIOAttempt'
        QtMocHelpers::SignalData<void(bool, bool, bool, int, QString)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 }, { QMetaType::Bool, 23 }, { QMetaType::Bool, 24 }, { QMetaType::Int, 25 },
            { QMetaType::QString, 26 },
        }}),
        // Signal 'eventCustomisation'
        QtMocHelpers::SignalData<void(quint32, bool, QString)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'finalSyncStarting'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'eventFinalSync'
        QtMocHelpers::SignalData<void(quint32, bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventVerify'
        QtMocHelpers::SignalData<void(quint32, bool, QByteArray, QByteArray)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 }, { QMetaType::QByteArray, 31 }, { QMetaType::QByteArray, 32 },
        }}),
        // Signal 'eventDecompressInit'
        QtMocHelpers::SignalData<void(quint32, bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventPeriodicSync'
        QtMocHelpers::SignalData<void(quint32, bool, quint64)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 }, { QMetaType::ULongLong, 35 },
        }}),
        // Signal 'eventImageExtraction'
        QtMocHelpers::SignalData<void(quint32, bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventPartitionTableWrite'
        QtMocHelpers::SignalData<void(quint32, bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventFatPartitionSetup'
        QtMocHelpers::SignalData<void(quint32, bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventDeviceClose'
        QtMocHelpers::SignalData<void(quint32, bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'eventNetworkRetry'
        QtMocHelpers::SignalData<void(quint32, QString)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 41 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'eventNetworkConnectionStats'
        QtMocHelpers::SignalData<void(QString)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'eventDeviceIOTimeout'
        QtMocHelpers::SignalData<void(quint32, QString)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 44 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'eventQueueDepthReduction'
        QtMocHelpers::SignalData<void(int, int, int)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 46 }, { QMetaType::Int, 47 }, { QMetaType::Int, 44 },
        }}),
        // Signal 'eventDrainAndHotSwap'
        QtMocHelpers::SignalData<void(quint32, int, bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::Int, 49 }, { QMetaType::Bool, 1 },
        }}),
        // Signal 'syncFallbackActivated'
        QtMocHelpers::SignalData<void(QString)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 },
        }}),
        // Signal 'requestWriteRestart'
        QtMocHelpers::SignalData<void(QString)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 },
        }}),
        // Signal 'eventWriteTimingBreakdown'
        QtMocHelpers::SignalData<void(quint32, quint64, quint64, quint64, quint64, quint32)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 54 }, { QMetaType::ULongLong, 55 }, { QMetaType::ULongLong, 56 }, { QMetaType::ULongLong, 57 },
            { QMetaType::ULongLong, 58 }, { QMetaType::UInt, 59 },
        }}),
        // Signal 'eventWriteSizeDistribution'
        QtMocHelpers::SignalData<void(quint32, quint32, quint32, quint64, quint32)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 61 }, { QMetaType::UInt, 62 }, { QMetaType::UInt, 63 }, { QMetaType::ULongLong, 64 },
            { QMetaType::UInt, 65 },
        }}),
        // Signal 'eventWriteAfterSyncImpact'
        QtMocHelpers::SignalData<void(quint32, quint32, quint32)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 67 }, { QMetaType::UInt, 68 }, { QMetaType::UInt, 69 },
        }}),
        // Signal 'eventAsyncIOConfig'
        QtMocHelpers::SignalData<void(bool, bool, int, quint32)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 71 }, { QMetaType::Bool, 72 }, { QMetaType::Int, 73 }, { QMetaType::UInt, 74 },
        }}),
        // Signal 'eventAsyncIOTiming'
        QtMocHelpers::SignalData<void(quint32, quint64, quint32)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 76 }, { QMetaType::ULongLong, 35 }, { QMetaType::UInt, 65 },
        }}),
        // Signal 'bottleneckStateChanged'
        QtMocHelpers::SignalData<void(DownloadThread::BottleneckState, quint32)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 78, 79 }, { QMetaType::UInt, 80 },
        }}),
        // Signal 'asyncWriteProgress'
        QtMocHelpers::SignalData<void(quint64, quint64)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 35 }, { QMetaType::ULongLong, 64 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'BottleneckState'
        QtMocHelpers::EnumData<enum BottleneckState>(82, 82, QMC::EnumIsScoped).add({
            {   83, BottleneckState::None },
            {   84, BottleneckState::Network },
            {   85, BottleneckState::Decompression },
            {   86, BottleneckState::Storage },
            {   87, BottleneckState::Verifying },
        }),
    };
    return QtMocHelpers::metaObjectData<DownloadThread, qt_meta_tag_ZN14DownloadThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DownloadThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DownloadThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DownloadThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DownloadThreadE_t>.metaTypes,
    nullptr
} };

void DownloadThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DownloadThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->success(); break;
        case 1: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->cacheFileUpdated((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: _t->cacheFileHashUpdated((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 4: _t->finalizing(); break;
        case 5: _t->preparationStatusUpdate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->eventDriveUnmount((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->eventDriveUnmountVolumes((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->eventDriveDiskClean((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->eventDriveRescan((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->eventDriveOpen((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 11: _t->eventDriveAuthorization((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->eventDriveMbrZeroing((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 13: _t->eventDirectIOAttempt((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 14: _t->eventCustomisation((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 15: _t->finalSyncStarting(); break;
        case 16: _t->eventFinalSync((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->eventVerify((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4]))); break;
        case 18: _t->eventDecompressInit((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->eventPeriodicSync((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3]))); break;
        case 20: _t->eventImageExtraction((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 21: _t->eventPartitionTableWrite((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->eventFatPartitionSetup((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 23: _t->eventDeviceClose((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 24: _t->eventNetworkRetry((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 25: _t->eventNetworkConnectionStats((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->eventDeviceIOTimeout((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 27: _t->eventQueueDepthReduction((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 28: _t->eventDrainAndHotSwap((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 29: _t->syncFallbackActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->requestWriteRestart((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->eventWriteTimingBreakdown((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[6]))); break;
        case 32: _t->eventWriteSizeDistribution((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[5]))); break;
        case 33: _t->eventWriteAfterSyncImpact((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3]))); break;
        case 34: _t->eventAsyncIOConfig((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[4]))); break;
        case 35: _t->eventAsyncIOTiming((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3]))); break;
        case 36: _t->bottleneckStateChanged((*reinterpret_cast<std::add_pointer_t<DownloadThread::BottleneckState>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2]))); break;
        case 37: _t->asyncWriteProgress((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)()>(_a, &DownloadThread::success, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QString )>(_a, &DownloadThread::error, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QByteArray )>(_a, &DownloadThread::cacheFileUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QByteArray , QByteArray )>(_a, &DownloadThread::cacheFileHashUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)()>(_a, &DownloadThread::finalizing, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QString )>(_a, &DownloadThread::preparationStatusUpdate, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDriveUnmount, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDriveUnmountVolumes, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDriveDiskClean, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDriveRescan, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool , QString )>(_a, &DownloadThread::eventDriveOpen, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDriveAuthorization, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool , QString )>(_a, &DownloadThread::eventDriveMbrZeroing, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(bool , bool , bool , int , QString )>(_a, &DownloadThread::eventDirectIOAttempt, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool , QString )>(_a, &DownloadThread::eventCustomisation, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)()>(_a, &DownloadThread::finalSyncStarting, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventFinalSync, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool , QByteArray , QByteArray )>(_a, &DownloadThread::eventVerify, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDecompressInit, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool , quint64 )>(_a, &DownloadThread::eventPeriodicSync, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventImageExtraction, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventPartitionTableWrite, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventFatPartitionSetup, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , bool )>(_a, &DownloadThread::eventDeviceClose, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , QString )>(_a, &DownloadThread::eventNetworkRetry, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QString )>(_a, &DownloadThread::eventNetworkConnectionStats, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , QString )>(_a, &DownloadThread::eventDeviceIOTimeout, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(int , int , int )>(_a, &DownloadThread::eventQueueDepthReduction, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , int , bool )>(_a, &DownloadThread::eventDrainAndHotSwap, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QString )>(_a, &DownloadThread::syncFallbackActivated, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(QString )>(_a, &DownloadThread::requestWriteRestart, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , quint64 , quint64 , quint64 , quint64 , quint32 )>(_a, &DownloadThread::eventWriteTimingBreakdown, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , quint32 , quint32 , quint64 , quint32 )>(_a, &DownloadThread::eventWriteSizeDistribution, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , quint32 , quint32 )>(_a, &DownloadThread::eventWriteAfterSyncImpact, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(bool , bool , int , quint32 )>(_a, &DownloadThread::eventAsyncIOConfig, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint32 , quint64 , quint32 )>(_a, &DownloadThread::eventAsyncIOTiming, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(DownloadThread::BottleneckState , quint32 )>(_a, &DownloadThread::bottleneckStateChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (DownloadThread::*)(quint64 , quint64 )>(_a, &DownloadThread::asyncWriteProgress, 37))
            return;
    }
}

const QMetaObject *DownloadThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DownloadThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DownloadThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int DownloadThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 38;
    }
    return _id;
}

// SIGNAL 0
void DownloadThread::success()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DownloadThread::error(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DownloadThread::cacheFileUpdated(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DownloadThread::cacheFileHashUpdated(QByteArray _t1, QByteArray _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void DownloadThread::finalizing()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DownloadThread::preparationStatusUpdate(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DownloadThread::eventDriveUnmount(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void DownloadThread::eventDriveUnmountVolumes(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void DownloadThread::eventDriveDiskClean(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void DownloadThread::eventDriveRescan(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void DownloadThread::eventDriveOpen(quint32 _t1, bool _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3);
}

// SIGNAL 11
void DownloadThread::eventDriveAuthorization(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void DownloadThread::eventDriveMbrZeroing(quint32 _t1, bool _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3);
}

// SIGNAL 13
void DownloadThread::eventDirectIOAttempt(bool _t1, bool _t2, bool _t3, int _t4, QString _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 14
void DownloadThread::eventCustomisation(quint32 _t1, bool _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2, _t3);
}

// SIGNAL 15
void DownloadThread::finalSyncStarting()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void DownloadThread::eventFinalSync(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void DownloadThread::eventVerify(quint32 _t1, bool _t2, QByteArray _t3, QByteArray _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 18
void DownloadThread::eventDecompressInit(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2);
}

// SIGNAL 19
void DownloadThread::eventPeriodicSync(quint32 _t1, bool _t2, quint64 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2, _t3);
}

// SIGNAL 20
void DownloadThread::eventImageExtraction(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1, _t2);
}

// SIGNAL 21
void DownloadThread::eventPartitionTableWrite(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void DownloadThread::eventFatPartitionSetup(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}

// SIGNAL 23
void DownloadThread::eventDeviceClose(quint32 _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1, _t2);
}

// SIGNAL 24
void DownloadThread::eventNetworkRetry(quint32 _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1, _t2);
}

// SIGNAL 25
void DownloadThread::eventNetworkConnectionStats(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void DownloadThread::eventDeviceIOTimeout(quint32 _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1, _t2);
}

// SIGNAL 27
void DownloadThread::eventQueueDepthReduction(int _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1, _t2, _t3);
}

// SIGNAL 28
void DownloadThread::eventDrainAndHotSwap(quint32 _t1, int _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1, _t2, _t3);
}

// SIGNAL 29
void DownloadThread::syncFallbackActivated(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void DownloadThread::requestWriteRestart(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void DownloadThread::eventWriteTimingBreakdown(quint32 _t1, quint64 _t2, quint64 _t3, quint64 _t4, quint64 _t5, quint32 _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 32
void DownloadThread::eventWriteSizeDistribution(quint32 _t1, quint32 _t2, quint32 _t3, quint64 _t4, quint32 _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 33
void DownloadThread::eventWriteAfterSyncImpact(quint32 _t1, quint32 _t2, quint32 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1, _t2, _t3);
}

// SIGNAL 34
void DownloadThread::eventAsyncIOConfig(bool _t1, bool _t2, int _t3, quint32 _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 35
void DownloadThread::eventAsyncIOTiming(quint32 _t1, quint64 _t2, quint32 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1, _t2, _t3);
}

// SIGNAL 36
void DownloadThread::bottleneckStateChanged(DownloadThread::BottleneckState _t1, quint32 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1, _t2);
}

// SIGNAL 37
void DownloadThread::asyncWriteProgress(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1, _t2);
}
QT_WARNING_POP
