/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // countries_generated.txt
  0x0,0x0,0x2,0x1f,
  0x41,
  0x44,0xa,0x41,0x45,0xa,0x41,0x46,0xa,0x41,0x49,0xa,0x41,0x4c,0xa,0x41,0x4d,
  0xa,0x41,0x4e,0xa,0x41,0x52,0xa,0x41,0x53,0xa,0x41,0x54,0xa,0x41,0x55,0xa,
  0x41,0x57,0xa,0x41,0x5a,0xa,0x42,0x41,0xa,0x42,0x42,0xa,0x42,0x44,0xa,0x42,
  0x45,0xa,0x42,0x46,0xa,0x42,0x47,0xa,0x42,0x48,0xa,0x42,0x4c,0xa,0x42,0x4d,
  0xa,0x42,0x4e,0xa,0x42,0x4f,0xa,0x42,0x52,0xa,0x42,0x53,0xa,0x42,0x54,0xa,
  0x42,0x57,0xa,0x42,0x59,0xa,0x42,0x5a,0xa,0x43,0x41,0xa,0x43,0x46,0xa,0x43,
  0x48,0xa,0x43,0x49,0xa,0x43,0x4c,0xa,0x43,0x4e,0xa,0x43,0x4f,0xa,0x43,0x52,
  0xa,0x43,0x55,0xa,0x43,0x58,0xa,0x43,0x59,0xa,0x43,0x5a,0xa,0x44,0x45,0xa,
  0x44,0x4b,0xa,0x44,0x4d,0xa,0x44,0x4f,0xa,0x44,0x5a,0xa,0x45,0x43,0xa,0x45,
  0x45,0xa,0x45,0x47,0xa,0x45,0x53,0xa,0x45,0x54,0xa,0x46,0x49,0xa,0x46,0x4d,
  0xa,0x46,0x4f,0xa,0x46,0x52,0xa,0x47,0x42,0xa,0x47,0x44,0xa,0x47,0x45,0xa,
  0x47,0x46,0xa,0x47,0x48,0xa,0x47,0x49,0xa,0x47,0x4c,0xa,0x47,0x50,0xa,0x47,
  0x52,0xa,0x47,0x54,0xa,0x47,0x55,0xa,0x47,0x59,0xa,0x48,0x4b,0xa,0x48,0x4e,
  0xa,0x48,0x52,0xa,0x48,0x54,0xa,0x48,0x55,0xa,0x49,0x44,0xa,0x49,0x45,0xa,
  0x49,0x4c,0xa,0x49,0x4d,0xa,0x49,0x4e,0xa,0x49,0x52,0xa,0x49,0x53,0xa,0x49,
  0x54,0xa,0x4a,0x4d,0xa,0x4a,0x4f,0xa,0x4a,0x50,0xa,0x4b,0x45,0xa,0x4b,0x48,
  0xa,0x4b,0x4e,0xa,0x4b,0x50,0xa,0x4b,0x52,0xa,0x4b,0x57,0xa,0x4b,0x59,0xa,
  0x4b,0x5a,0xa,0x4c,0x42,0xa,0x4c,0x43,0xa,0x4c,0x49,0xa,0x4c,0x4b,0xa,0x4c,
  0x53,0xa,0x4c,0x54,0xa,0x4c,0x55,0xa,0x4c,0x56,0xa,0x4d,0x41,0xa,0x4d,0x43,
  0xa,0x4d,0x44,0xa,0x4d,0x45,0xa,0x4d,0x46,0xa,0x4d,0x48,0xa,0x4d,0x4b,0xa,
  0x4d,0x4e,0xa,0x4d,0x4f,0xa,0x4d,0x50,0xa,0x4d,0x51,0xa,0x4d,0x52,0xa,0x4d,
  0x54,0xa,0x4d,0x55,0xa,0x4d,0x56,0xa,0x4d,0x57,0xa,0x4d,0x58,0xa,0x4d,0x59,
  0xa,0x4e,0x41,0xa,0x4e,0x47,0xa,0x4e,0x49,0xa,0x4e,0x4c,0xa,0x4e,0x4f,0xa,
  0x4e,0x50,0xa,0x4e,0x5a,0xa,0x4f,0x4d,0xa,0x50,0x41,0xa,0x50,0x45,0xa,0x50,
  0x46,0xa,0x50,0x47,0xa,0x50,0x48,0xa,0x50,0x4b,0xa,0x50,0x4c,0xa,0x50,0x4d,
  0xa,0x50,0x52,0xa,0x50,0x54,0xa,0x50,0x57,0xa,0x50,0x59,0xa,0x51,0x41,0xa,
  0x52,0x45,0xa,0x52,0x4f,0xa,0x52,0x53,0xa,0x52,0x55,0xa,0x52,0x57,0xa,0x53,
  0x41,0xa,0x53,0x45,0xa,0x53,0x47,0xa,0x53,0x49,0xa,0x53,0x4b,0xa,0x53,0x4d,
  0xa,0x53,0x4e,0xa,0x53,0x52,0xa,0x53,0x56,0xa,0x53,0x58,0xa,0x53,0x59,0xa,
  0x54,0x43,0xa,0x54,0x44,0xa,0x54,0x47,0xa,0x54,0x48,0xa,0x54,0x4e,0xa,0x54,
  0x52,0xa,0x54,0x54,0xa,0x54,0x57,0xa,0x54,0x5a,0xa,0x55,0x41,0xa,0x55,0x47,
  0xa,0x55,0x53,0xa,0x55,0x59,0xa,0x55,0x5a,0xa,0x56,0x41,0xa,0x56,0x43,0xa,
  0x56,0x45,0xa,0x56,0x49,0xa,0x56,0x4e,0xa,0x56,0x55,0xa,0x57,0x46,0xa,0x57,
  0x53,0xa,0x59,0x45,0xa,0x59,0x54,0xa,0x5a,0x41,0xa,0x5a,0x57,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // countries.txt
  0x0,0xd,
  0xb,0x1e,0xed,0x54,
  0x0,0x63,
  0x0,0x6f,0x0,0x75,0x0,0x6e,0x0,0x74,0x0,0x72,0x0,0x69,0x0,0x65,0x0,0x73,0x0,0x2e,0x0,0x74,0x0,0x78,0x0,0x74,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,
  // :/countries.txt
  0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9d,0x7b,0xdd,0xb2,0xa0,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_countries_generated)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_countries_generated)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_countries_generated)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_countries_generated)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_countries_generated)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_countries_generated)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
